# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["MonetaryPercentageDiscountAdjustment", "Filter"]


class Filter(BaseModel):
    field: Literal["price_id", "item_id", "price_type", "currency", "pricing_unit_id"]
    """The property of the price to filter on."""

    operator: Literal["includes", "excludes"]
    """Should prices that match the filter be included or excluded."""

    values: List[str]
    """The IDs or values that match this filter."""


class MonetaryPercentageDiscountAdjustment(BaseModel):
    id: str

    adjustment_type: Literal["percentage_discount"]

    amount: str
    """The value applied by an adjustment."""

    applies_to_price_ids: List[str]
    """The price IDs that this adjustment applies to."""

    filters: List[Filter]
    """The filters that determine which prices to apply this adjustment to."""

    is_invoice_level: bool
    """
    True for adjustments that apply to an entire invoice, false for adjustments that
    apply to only one price.
    """

    percentage_discount: float
    """
    The percentage (as a value between 0 and 1) by which to discount the price
    intervals this adjustment applies to in a given billing period.
    """

    reason: Optional[str] = None
    """The reason for the adjustment."""

    replaces_adjustment_id: Optional[str] = None
    """The adjustment id this adjustment replaces.

    This adjustment will take the place of the replaced adjustment in plan version
    migrations.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Optional
from datetime import date

import httpx

from .. import _legacy_response
from ..types import invoice_line_item_create_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import to_streamed_response_wrapper, async_to_streamed_response_wrapper
from .._base_client import make_request_options
from ..types.invoice_line_item_create_response import InvoiceLineItemCreateResponse

__all__ = ["InvoiceLineItems", "AsyncInvoiceLineItems"]


class InvoiceLineItems(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> InvoiceLineItemsWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/orbcorp/orb-python#accessing-raw-response-data-eg-headers
        """
        return InvoiceLineItemsWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> InvoiceLineItemsWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/orbcorp/orb-python#with_streaming_response
        """
        return InvoiceLineItemsWithStreamingResponse(self)

    def create(
        self,
        *,
        amount: str,
        end_date: Union[str, date],
        invoice_id: str,
        quantity: float,
        start_date: Union[str, date],
        item_id: Optional[str] | Omit = omit,
        name: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
        idempotency_key: str | None = None,
    ) -> InvoiceLineItemCreateResponse:
        """This creates a one-off fixed fee invoice line item on an Invoice.

        This can only
        be done for invoices that are in a `draft` status.

        The behavior depends on which parameters are provided:

        - If `item_id` is provided without `name`: The item is looked up by ID, and the
          item's name is used for the line item.
        - If `name` is provided without `item_id`: An item with the given name is
          searched for in the account. If found, that item is used. If not found, a new
          item is created with that name. The new item's name is used for the line item.
        - If both `item_id` and `name` are provided: The item is looked up by ID for
          association, but the provided `name` is used for the line item (not the item's
          name).

        Args:
          amount: The total amount in the invoice's currency to add to the line item.

          end_date: A date string to specify the line item's end date in the customer's timezone.

          invoice_id: The id of the Invoice to add this line item.

          quantity: The number of units on the line item

          start_date: A date string to specify the line item's start date in the customer's timezone.

          item_id: The id of the item to associate with this line item. If provided without `name`,
              the item's name will be used for the price/line item. If provided with `name`,
              the item will be associated but `name` will be used for the line item. At least
              one of `name` or `item_id` must be provided.

          name: The name to use for the line item. If `item_id` is not provided, Orb will search
              for an item with this name. If found, that item will be associated with the line
              item. If not found, a new item will be created with this name. If `item_id` is
              provided, this name will be used for the line item, but the item association
              will be based on `item_id`. At least one of `name` or `item_id` must be
              provided.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        return self._post(
            "/invoice_line_items",
            body=maybe_transform(
                {
                    "amount": amount,
                    "end_date": end_date,
                    "invoice_id": invoice_id,
                    "quantity": quantity,
                    "start_date": start_date,
                    "item_id": item_id,
                    "name": name,
                },
                invoice_line_item_create_params.InvoiceLineItemCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=InvoiceLineItemCreateResponse,
        )


class AsyncInvoiceLineItems(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncInvoiceLineItemsWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/orbcorp/orb-python#accessing-raw-response-data-eg-headers
        """
        return AsyncInvoiceLineItemsWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncInvoiceLineItemsWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/orbcorp/orb-python#with_streaming_response
        """
        return AsyncInvoiceLineItemsWithStreamingResponse(self)

    async def create(
        self,
        *,
        amount: str,
        end_date: Union[str, date],
        invoice_id: str,
        quantity: float,
        start_date: Union[str, date],
        item_id: Optional[str] | Omit = omit,
        name: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
        idempotency_key: str | None = None,
    ) -> InvoiceLineItemCreateResponse:
        """This creates a one-off fixed fee invoice line item on an Invoice.

        This can only
        be done for invoices that are in a `draft` status.

        The behavior depends on which parameters are provided:

        - If `item_id` is provided without `name`: The item is looked up by ID, and the
          item's name is used for the line item.
        - If `name` is provided without `item_id`: An item with the given name is
          searched for in the account. If found, that item is used. If not found, a new
          item is created with that name. The new item's name is used for the line item.
        - If both `item_id` and `name` are provided: The item is looked up by ID for
          association, but the provided `name` is used for the line item (not the item's
          name).

        Args:
          amount: The total amount in the invoice's currency to add to the line item.

          end_date: A date string to specify the line item's end date in the customer's timezone.

          invoice_id: The id of the Invoice to add this line item.

          quantity: The number of units on the line item

          start_date: A date string to specify the line item's start date in the customer's timezone.

          item_id: The id of the item to associate with this line item. If provided without `name`,
              the item's name will be used for the price/line item. If provided with `name`,
              the item will be associated but `name` will be used for the line item. At least
              one of `name` or `item_id` must be provided.

          name: The name to use for the line item. If `item_id` is not provided, Orb will search
              for an item with this name. If found, that item will be associated with the line
              item. If not found, a new item will be created with this name. If `item_id` is
              provided, this name will be used for the line item, but the item association
              will be based on `item_id`. At least one of `name` or `item_id` must be
              provided.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        return await self._post(
            "/invoice_line_items",
            body=await async_maybe_transform(
                {
                    "amount": amount,
                    "end_date": end_date,
                    "invoice_id": invoice_id,
                    "quantity": quantity,
                    "start_date": start_date,
                    "item_id": item_id,
                    "name": name,
                },
                invoice_line_item_create_params.InvoiceLineItemCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=InvoiceLineItemCreateResponse,
        )


class InvoiceLineItemsWithRawResponse:
    def __init__(self, invoice_line_items: InvoiceLineItems) -> None:
        self._invoice_line_items = invoice_line_items

        self.create = _legacy_response.to_raw_response_wrapper(
            invoice_line_items.create,
        )


class AsyncInvoiceLineItemsWithRawResponse:
    def __init__(self, invoice_line_items: AsyncInvoiceLineItems) -> None:
        self._invoice_line_items = invoice_line_items

        self.create = _legacy_response.async_to_raw_response_wrapper(
            invoice_line_items.create,
        )


class InvoiceLineItemsWithStreamingResponse:
    def __init__(self, invoice_line_items: InvoiceLineItems) -> None:
        self._invoice_line_items = invoice_line_items

        self.create = to_streamed_response_wrapper(
            invoice_line_items.create,
        )


class AsyncInvoiceLineItemsWithStreamingResponse:
    def __init__(self, invoice_line_items: AsyncInvoiceLineItems) -> None:
        self._invoice_line_items = invoice_line_items

        self.create = async_to_streamed_response_wrapper(
            invoice_line_items.create,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["AmountDiscount", "Filter"]


class Filter(BaseModel):
    field: Literal["price_id", "item_id", "price_type", "currency", "pricing_unit_id"]
    """The property of the price to filter on."""

    operator: Literal["includes", "excludes"]
    """Should prices that match the filter be included or excluded."""

    values: List[str]
    """The IDs or values that match this filter."""


class AmountDiscount(BaseModel):
    amount_discount: str
    """Only available if discount_type is `amount`."""

    discount_type: Literal["amount"]

    applies_to_price_ids: Optional[List[str]] = None
    """List of price_ids that this discount applies to.

    For plan/plan phase discounts, this can be a subset of prices.
    """

    filters: Optional[List[Filter]] = None
    """The filters that determine which prices to apply this discount to."""

    reason: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Optional
from datetime import date
from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["SubscriptionChangeApplyParams"]


class SubscriptionChangeApplyParams(TypedDict, total=False):
    description: Optional[str]
    """Description to apply to the balance transaction representing this credit."""

    mark_as_paid: Optional[bool]
    """Mark all pending invoices that are payable as paid.

    If amount is also provided, mark as paid and credit the difference to the
    customer's balance.
    """

    payment_external_id: Optional[str]
    """An optional external ID to associate with the payment.

    Only applicable when mark_as_paid is true.
    """

    payment_notes: Optional[str]
    """Optional notes about the payment. Only applicable when mark_as_paid is true."""

    payment_received_date: Annotated[Union[str, date, None], PropertyInfo(format="iso8601")]
    """A date string to specify the date the payment was received.

    Only applicable when mark_as_paid is true. If not provided, defaults to the
    current date.
    """

    previously_collected_amount: Optional[str]
    """Amount already collected to apply to the customer's balance.

    If mark_as_paid is also provided, credit the difference to the customer's
    balance.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ..._models import BaseModel

__all__ = ["ItemSlim"]


class ItemSlim(BaseModel):
    id: str
    """The Orb-assigned unique identifier for the item."""

    name: str
    """The name of the item."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable, Optional
from typing_extensions import Literal, Required, TypedDict

from ..._types import SequenceNotStr
from .custom_expiration import CustomExpiration

__all__ = ["NewAllocationPrice", "Filter"]


class Filter(TypedDict, total=False):
    field: Required[Literal["item_id"]]
    """The property of the price the block applies to. Only item_id is supported."""

    operator: Required[Literal["includes", "excludes"]]
    """Should prices that match the filter be included or excluded."""

    values: Required[SequenceNotStr[str]]
    """The IDs or values that match this filter."""


class NewAllocationPrice(TypedDict, total=False):
    amount: Required[str]
    """An amount of the currency to allocate to the customer at the specified cadence."""

    cadence: Required[Literal["one_time", "monthly", "quarterly", "semi_annual", "annual"]]
    """The cadence at which to allocate the amount to the customer."""

    currency: Required[str]
    """
    An ISO 4217 currency string or a custom pricing unit identifier in which to bill
    this price.
    """

    custom_expiration: Optional[CustomExpiration]
    """The custom expiration for the allocation."""

    expires_at_end_of_cadence: Optional[bool]
    """
    Whether the allocated amount should expire at the end of the cadence or roll
    over to the next period. Set to null if using custom_expiration.
    """

    filters: Optional[Iterable[Filter]]
    """The filters that determine which items the allocation applies to."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable, Optional
from typing_extensions import Literal, Required, TypedDict

from ..._types import SequenceNotStr

__all__ = ["AmountDiscount", "Filter"]


class Filter(TypedDict, total=False):
    field: Required[Literal["price_id", "item_id", "price_type", "currency", "pricing_unit_id"]]
    """The property of the price to filter on."""

    operator: Required[Literal["includes", "excludes"]]
    """Should prices that match the filter be included or excluded."""

    values: Required[SequenceNotStr[str]]
    """The IDs or values that match this filter."""


class AmountDiscount(TypedDict, total=False):
    amount_discount: Required[str]
    """Only available if discount_type is `amount`."""

    discount_type: Required[Literal["amount"]]

    applies_to_price_ids: Optional[SequenceNotStr[str]]
    """List of price_ids that this discount applies to.

    For plan/plan phase discounts, this can be a subset of prices.
    """

    filters: Optional[Iterable[Filter]]
    """The filters that determine which prices to apply this discount to."""

    reason: Optional[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Optional
from datetime import date
from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["InvoiceLineItemCreateParams"]


class InvoiceLineItemCreateParams(TypedDict, total=False):
    amount: Required[str]
    """The total amount in the invoice's currency to add to the line item."""

    end_date: Required[Annotated[Union[str, date], PropertyInfo(format="iso8601")]]
    """A date string to specify the line item's end date in the customer's timezone."""

    invoice_id: Required[str]
    """The id of the Invoice to add this line item."""

    quantity: Required[float]
    """The number of units on the line item"""

    start_date: Required[Annotated[Union[str, date], PropertyInfo(format="iso8601")]]
    """A date string to specify the line item's start date in the customer's timezone."""

    item_id: Optional[str]
    """The id of the item to associate with this line item.

    If provided without `name`, the item's name will be used for the price/line
    item. If provided with `name`, the item will be associated but `name` will be
    used for the line item. At least one of `name` or `item_id` must be provided.
    """

    name: Optional[str]
    """The name to use for the line item.

    If `item_id` is not provided, Orb will search for an item with this name. If
    found, that item will be associated with the line item. If not found, a new item
    will be created with this name. If `item_id` is provided, this name will be used
    for the line item, but the item association will be based on `item_id`. At least
    one of `name` or `item_id` must be provided.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["UnitConfig"]


class UnitConfig(TypedDict, total=False):
    unit_amount: Required[str]
    """Rate per unit of usage"""

    prorated: bool
    """If true, subtotals from this price are prorated based on the service period"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["PercentageDiscountInterval", "Filter"]


class Filter(BaseModel):
    field: Literal["price_id", "item_id", "price_type", "currency", "pricing_unit_id"]
    """The property of the price to filter on."""

    operator: Literal["includes", "excludes"]
    """Should prices that match the filter be included or excluded."""

    values: List[str]
    """The IDs or values that match this filter."""


class PercentageDiscountInterval(BaseModel):
    applies_to_price_interval_ids: List[str]
    """The price interval ids that this discount interval applies to."""

    discount_type: Literal["percentage"]

    end_date: Optional[datetime] = None
    """The end date of the discount interval."""

    filters: List[Filter]
    """The filters that determine which prices this discount interval applies to."""

    percentage_discount: float
    """
    Only available if discount_type is `percentage`.This is a number between 0
    and 1.
    """

    start_date: datetime
    """The start date of the discount interval."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ..._models import BaseModel
from .custom_expiration import CustomExpiration

__all__ = ["Allocation", "Filter"]


class Filter(BaseModel):
    field: Literal["price_id", "item_id", "price_type", "currency", "pricing_unit_id"]
    """The property of the price to filter on."""

    operator: Literal["includes", "excludes"]
    """Should prices that match the filter be included or excluded."""

    values: List[str]
    """The IDs or values that match this filter."""


class Allocation(BaseModel):
    allows_rollover: bool

    currency: str

    custom_expiration: Optional[CustomExpiration] = None

    filters: Optional[List[Filter]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from orb import Orb, AsyncOrb
from orb._utils import parse_datetime
from tests.utils import assert_matches_type
from orb.pagination import SyncPage, AsyncPage
from orb.types.customers import (
    BalanceTransactionListResponse,
    BalanceTransactionCreateResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestBalanceTransactions:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: Orb) -> None:
        balance_transaction = client.customers.balance_transactions.create(
            customer_id="customer_id",
            amount="amount",
            type="increment",
        )
        assert_matches_type(BalanceTransactionCreateResponse, balance_transaction, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: Orb) -> None:
        balance_transaction = client.customers.balance_transactions.create(
            customer_id="customer_id",
            amount="amount",
            type="increment",
            description="description",
        )
        assert_matches_type(BalanceTransactionCreateResponse, balance_transaction, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: Orb) -> None:
        response = client.customers.balance_transactions.with_raw_response.create(
            customer_id="customer_id",
            amount="amount",
            type="increment",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        balance_transaction = response.parse()
        assert_matches_type(BalanceTransactionCreateResponse, balance_transaction, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: Orb) -> None:
        with client.customers.balance_transactions.with_streaming_response.create(
            customer_id="customer_id",
            amount="amount",
            type="increment",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            balance_transaction = response.parse()
            assert_matches_type(BalanceTransactionCreateResponse, balance_transaction, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_create(self, client: Orb) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `customer_id` but received ''"):
            client.customers.balance_transactions.with_raw_response.create(
                customer_id="",
                amount="amount",
                type="increment",
            )

    @parametrize
    def test_method_list(self, client: Orb) -> None:
        balance_transaction = client.customers.balance_transactions.list(
            customer_id="customer_id",
        )
        assert_matches_type(SyncPage[BalanceTransactionListResponse], balance_transaction, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: Orb) -> None:
        balance_transaction = client.customers.balance_transactions.list(
            customer_id="customer_id",
            cursor="cursor",
            limit=1,
            operation_time_gt=parse_datetime("2019-12-27T18:11:19.117Z"),
            operation_time_gte=parse_datetime("2019-12-27T18:11:19.117Z"),
            operation_time_lt=parse_datetime("2019-12-27T18:11:19.117Z"),
            operation_time_lte=parse_datetime("2019-12-27T18:11:19.117Z"),
        )
        assert_matches_type(SyncPage[BalanceTransactionListResponse], balance_transaction, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: Orb) -> None:
        response = client.customers.balance_transactions.with_raw_response.list(
            customer_id="customer_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        balance_transaction = response.parse()
        assert_matches_type(SyncPage[BalanceTransactionListResponse], balance_transaction, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: Orb) -> None:
        with client.customers.balance_transactions.with_streaming_response.list(
            customer_id="customer_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            balance_transaction = response.parse()
            assert_matches_type(SyncPage[BalanceTransactionListResponse], balance_transaction, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_list(self, client: Orb) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `customer_id` but received ''"):
            client.customers.balance_transactions.with_raw_response.list(
                customer_id="",
            )


class TestAsyncBalanceTransactions:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_create(self, async_client: AsyncOrb) -> None:
        balance_transaction = await async_client.customers.balance_transactions.create(
            customer_id="customer_id",
            amount="amount",
            type="increment",
        )
        assert_matches_type(BalanceTransactionCreateResponse, balance_transaction, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncOrb) -> None:
        balance_transaction = await async_client.customers.balance_transactions.create(
            customer_id="customer_id",
            amount="amount",
            type="increment",
            description="description",
        )
        assert_matches_type(BalanceTransactionCreateResponse, balance_transaction, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncOrb) -> None:
        response = await async_client.customers.balance_transactions.with_raw_response.create(
            customer_id="customer_id",
            amount="amount",
            type="increment",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        balance_transaction = response.parse()
        assert_matches_type(BalanceTransactionCreateResponse, balance_transaction, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncOrb) -> None:
        async with async_client.customers.balance_transactions.with_streaming_response.create(
            customer_id="customer_id",
            amount="amount",
            type="increment",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            balance_transaction = await response.parse()
            assert_matches_type(BalanceTransactionCreateResponse, balance_transaction, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_create(self, async_client: AsyncOrb) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `customer_id` but received ''"):
            await async_client.customers.balance_transactions.with_raw_response.create(
                customer_id="",
                amount="amount",
                type="increment",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncOrb) -> None:
        balance_transaction = await async_client.customers.balance_transactions.list(
            customer_id="customer_id",
        )
        assert_matches_type(AsyncPage[BalanceTransactionListResponse], balance_transaction, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncOrb) -> None:
        balance_transaction = await async_client.customers.balance_transactions.list(
            customer_id="customer_id",
            cursor="cursor",
            limit=1,
            operation_time_gt=parse_datetime("2019-12-27T18:11:19.117Z"),
            operation_time_gte=parse_datetime("2019-12-27T18:11:19.117Z"),
            operation_time_lt=parse_datetime("2019-12-27T18:11:19.117Z"),
            operation_time_lte=parse_datetime("2019-12-27T18:11:19.117Z"),
        )
        assert_matches_type(AsyncPage[BalanceTransactionListResponse], balance_transaction, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncOrb) -> None:
        response = await async_client.customers.balance_transactions.with_raw_response.list(
            customer_id="customer_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        balance_transaction = response.parse()
        assert_matches_type(AsyncPage[BalanceTransactionListResponse], balance_transaction, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncOrb) -> None:
        async with async_client.customers.balance_transactions.with_streaming_response.list(
            customer_id="customer_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            balance_transaction = await response.parse()
            assert_matches_type(AsyncPage[BalanceTransactionListResponse], balance_transaction, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_list(self, async_client: AsyncOrb) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `customer_id` but received ''"):
            await async_client.customers.balance_transactions.with_raw_response.list(
                customer_id="",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel
from .sub_line_item_grouping import SubLineItemGrouping
from .sub_line_item_matrix_config import SubLineItemMatrixConfig

__all__ = ["MatrixSubLineItem"]


class MatrixSubLineItem(BaseModel):
    amount: str
    """The total amount for this sub line item."""

    grouping: Optional[SubLineItemGrouping] = None

    matrix_config: SubLineItemMatrixConfig

    name: str

    quantity: float

    type: Literal["matrix"]

    scaled_quantity: Optional[float] = None
    """The scaled quantity for this line item for specific pricing structures"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["UnitConfig"]


class UnitConfig(BaseModel):
    unit_amount: str
    """Rate per unit of usage"""

    prorated: Optional[bool] = None
    """If true, subtotals from this price are prorated based on the service period"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from orb import Orb, AsyncOrb
from orb.types import DimensionalPriceGroup
from tests.utils import assert_matches_type

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestExternalDimensionalPriceGroupID:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_retrieve(self, client: Orb) -> None:
        external_dimensional_price_group_id = (
            client.dimensional_price_groups.external_dimensional_price_group_id.retrieve(
                "external_dimensional_price_group_id",
            )
        )
        assert_matches_type(DimensionalPriceGroup, external_dimensional_price_group_id, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: Orb) -> None:
        response = client.dimensional_price_groups.external_dimensional_price_group_id.with_raw_response.retrieve(
            "external_dimensional_price_group_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        external_dimensional_price_group_id = response.parse()
        assert_matches_type(DimensionalPriceGroup, external_dimensional_price_group_id, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: Orb) -> None:
        with client.dimensional_price_groups.external_dimensional_price_group_id.with_streaming_response.retrieve(
            "external_dimensional_price_group_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            external_dimensional_price_group_id = response.parse()
            assert_matches_type(DimensionalPriceGroup, external_dimensional_price_group_id, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: Orb) -> None:
        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `external_dimensional_price_group_id` but received ''"
        ):
            client.dimensional_price_groups.external_dimensional_price_group_id.with_raw_response.retrieve(
                "",
            )

    @parametrize
    def test_method_update(self, client: Orb) -> None:
        external_dimensional_price_group_id = (
            client.dimensional_price_groups.external_dimensional_price_group_id.update(
                path_external_dimensional_price_group_id="external_dimensional_price_group_id",
            )
        )
        assert_matches_type(DimensionalPriceGroup, external_dimensional_price_group_id, path=["response"])

    @parametrize
    def test_method_update_with_all_params(self, client: Orb) -> None:
        external_dimensional_price_group_id = (
            client.dimensional_price_groups.external_dimensional_price_group_id.update(
                path_external_dimensional_price_group_id="external_dimensional_price_group_id",
                body_external_dimensional_price_group_id="external_dimensional_price_group_id",
                metadata={"foo": "string"},
            )
        )
        assert_matches_type(DimensionalPriceGroup, external_dimensional_price_group_id, path=["response"])

    @parametrize
    def test_raw_response_update(self, client: Orb) -> None:
        response = client.dimensional_price_groups.external_dimensional_price_group_id.with_raw_response.update(
            path_external_dimensional_price_group_id="external_dimensional_price_group_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        external_dimensional_price_group_id = response.parse()
        assert_matches_type(DimensionalPriceGroup, external_dimensional_price_group_id, path=["response"])

    @parametrize
    def test_streaming_response_update(self, client: Orb) -> None:
        with client.dimensional_price_groups.external_dimensional_price_group_id.with_streaming_response.update(
            path_external_dimensional_price_group_id="external_dimensional_price_group_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            external_dimensional_price_group_id = response.parse()
            assert_matches_type(DimensionalPriceGroup, external_dimensional_price_group_id, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update(self, client: Orb) -> None:
        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `path_external_dimensional_price_group_id` but received ''",
        ):
            client.dimensional_price_groups.external_dimensional_price_group_id.with_raw_response.update(
                path_external_dimensional_price_group_id="",
            )


class TestAsyncExternalDimensionalPriceGroupID:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncOrb) -> None:
        external_dimensional_price_group_id = (
            await async_client.dimensional_price_groups.external_dimensional_price_group_id.retrieve(
                "external_dimensional_price_group_id",
            )
        )
        assert_matches_type(DimensionalPriceGroup, external_dimensional_price_group_id, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncOrb) -> None:
        response = (
            await async_client.dimensional_price_groups.external_dimensional_price_group_id.with_raw_response.retrieve(
                "external_dimensional_price_group_id",
            )
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        external_dimensional_price_group_id = response.parse()
        assert_matches_type(DimensionalPriceGroup, external_dimensional_price_group_id, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncOrb) -> None:
        async with (
            async_client.dimensional_price_groups.external_dimensional_price_group_id.with_streaming_response.retrieve(
                "external_dimensional_price_group_id",
            )
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            external_dimensional_price_group_id = await response.parse()
            assert_matches_type(DimensionalPriceGroup, external_dimensional_price_group_id, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncOrb) -> None:
        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `external_dimensional_price_group_id` but received ''"
        ):
            await async_client.dimensional_price_groups.external_dimensional_price_group_id.with_raw_response.retrieve(
                "",
            )

    @parametrize
    async def test_method_update(self, async_client: AsyncOrb) -> None:
        external_dimensional_price_group_id = (
            await async_client.dimensional_price_groups.external_dimensional_price_group_id.update(
                path_external_dimensional_price_group_id="external_dimensional_price_group_id",
            )
        )
        assert_matches_type(DimensionalPriceGroup, external_dimensional_price_group_id, path=["response"])

    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncOrb) -> None:
        external_dimensional_price_group_id = (
            await async_client.dimensional_price_groups.external_dimensional_price_group_id.update(
                path_external_dimensional_price_group_id="external_dimensional_price_group_id",
                body_external_dimensional_price_group_id="external_dimensional_price_group_id",
                metadata={"foo": "string"},
            )
        )
        assert_matches_type(DimensionalPriceGroup, external_dimensional_price_group_id, path=["response"])

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncOrb) -> None:
        response = (
            await async_client.dimensional_price_groups.external_dimensional_price_group_id.with_raw_response.update(
                path_external_dimensional_price_group_id="external_dimensional_price_group_id",
            )
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        external_dimensional_price_group_id = response.parse()
        assert_matches_type(DimensionalPriceGroup, external_dimensional_price_group_id, path=["response"])

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncOrb) -> None:
        async with (
            async_client.dimensional_price_groups.external_dimensional_price_group_id.with_streaming_response.update(
                path_external_dimensional_price_group_id="external_dimensional_price_group_id",
            )
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            external_dimensional_price_group_id = await response.parse()
            assert_matches_type(DimensionalPriceGroup, external_dimensional_price_group_id, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update(self, async_client: AsyncOrb) -> None:
        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `path_external_dimensional_price_group_id` but received ''",
        ):
            await async_client.dimensional_price_groups.external_dimensional_price_group_id.with_raw_response.update(
                path_external_dimensional_price_group_id="",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["CreditListByExternalIDResponse", "Filter"]


class Filter(BaseModel):
    field: Literal["item_id"]
    """The property of the price the block applies to. Only item_id is supported."""

    operator: Literal["includes", "excludes"]
    """Should prices that match the filter be included or excluded."""

    values: List[str]
    """The IDs or values that match this filter."""


class CreditListByExternalIDResponse(BaseModel):
    id: str

    balance: float

    effective_date: Optional[datetime] = None

    expiry_date: Optional[datetime] = None

    filters: List[Filter]

    maximum_initial_balance: Optional[float] = None

    per_unit_cost_basis: Optional[str] = None

    status: Literal["active", "pending_payment"]
